#!/bin/sh
#
# Copyright (c) 1999-2004 SWsoft Inc.
# All rights reserved
#

#
# Plesk script
#


## @@constructor set_apache_params

remove_tmp_state()
{
	if [ -d "/tmp/.state" ]; then
		rm -Rf "/tmp/.state" >> $product_log 2>&1
	fi
}

# Print an error message and exit the program.
error()
{
    die "Exiting due to an error!  Please fix the error and try again." 
}

p_echo()
{
echo "$*" >> "$product_log" 2>&1
echo "$*"
}

pnnl_echo()
{
if [ -z $necho_defined ]; then
	# set up echo
	case "`echo 'x\c'`" in
    	'x\c')
			echo="echo"
			echo_keys=" -n"
			nnl=
		;;
		x)
			echo="echo"
			echo_keys=
			nnl="\c"
		;;
		*)
			echo "ERROR: unable to set up echo."
		;;
	esac
	necho_defined=true
fi

${echo} ${echo_keys} "$*${nnl}" >> "$product_log" 2>&1
${echo} ${echo_keys} "$*${nnl}"
}

keyupdate_failure_report()
{
	if test -z "$KEYUPDATE_FAILURE_INFO"; then
		return
	fi

	p_echo
	p_echo " *****************************************************************************"
	p_echo " *                                                                           *"
	p_echo " *  ERROR:      The $PRODUCT_NAME license failed to upgrade. The failure     *"
	p_echo " *              follows:                                                     *"
	p_echo
	p_echo "$KEYUPDATE_FAILURE_INFO"
	p_echo
	p_echo " *              If you did not request the automatic key upgrade,            *"
	p_echo " *              please contact SWsoft, Inc sales department for assistance:  *"
	p_echo " *              ${sales_email}. If you want to proceed with upgrade without *"
	p_echo " *              the license upgrade, please create an empty file             *"
	p_echo " *				$prodkey                                             *"
	p_echo " *                                                                           *"
	p_echo " *****************************************************************************"
	p_echo
}

die()
{

	if [ "X$trigger_uninstall" != "X1" ]; then
		printf "\a\a"
		p_echo
		p_echo "ERROR while trying to $*"
		echo "Check the error reason(see log file: ${product_log}), fix and try again"
		p_echo
		if [ "X$do_patch" != "X1" -a "X$do_reconfigure" != "X1" ]; then
			p_echo "Aborting..."
			p_echo
		fi
	fi

	smart_undo_install

	keyupdate_failure_report

	exit 1

}

err()
{

	if [ "X$trigger_uninstall" != "X1" ]; then
		p_echo
		p_echo "ERROR!"
		p_echo "Check the error reason(see log file: ${product_log}), fix and try again"
		p_echo
		if [ "X$do_patch" != "X1" -a "X$do_reconfigure" != "X1" ]; then
			p_echo "Exiting..."
			p_echo
		fi
	fi

	smart_undo_install

	exit 1
}
# This function needs for some print warning when some services cannot restart
# if called with arg, then we add arg to variable SR_WARNING and print warning
# message for arg, else print warning message for SR_WARNING
service_restart_warning(){
	if [ "X$1" != "X" ]; then
		if [ "X${SR_WARNING}" == "X" ]; then
			SR_WARNING=$1
		else
			for i in ${SR_WARNING}; do
				if [ "$i" == "$1" ]; then
					break
				fi
			done
			if [ "$1" != "$i" ]; then
				SR_WARNING=$SR_WARNING" "$1
			fi
		fi
		p_echo "WARNING!"
		p_echo "Some problems occured when $1 restart, check it after installation/upgrade complete"
	        p_echo
	 	p_echo "Continue..."
		p_echo
	else
		if [ "X${SR_WARNING}" != "X" ]; then
			p_echo "WARNING!"
		        p_echo "Some problems occured when following services restart:"
			for i in ${SR_WARNING}; do
				p_echo $i
			done
			p_echo "Don't forget to check it after installation/upgrade complete"
        		p_echo
	       	        p_echo "More information can be found in log file: ${product_log}"
			p_echo
		fi
	fi
}

warn()
{
inten=$1
p_echo
p_echo "WARNING!"
p_echo "During the $inten found some small problems(see log file: ${product_log})"
p_echo
p_echo "Continue..."
p_echo

}

report_problem()
{
	p_echo "***** $process minor problem report *****"
	echo "***** $process minor problem report *****" >> "$product_problems_log" 2>&1

	p_echo "$1"
	echo "$1" >> "$product_problems_log" 2>&1

	p_echo "$2"
	echo "$2" >> "$product_problems_log" 2>&1

	problems_occured=1
}


echo_try()
{
	msg="$*"
	pnnl_echo " Trying to $msg... "
}

suc()
{
	p_echo "done"
}


mk_backup()
{
	target="$1"
	dup="$2"
	opts="$3"

	if [ -L "$target" ]; then
		rm "$target"
	elif [ -$opts "$target" ]; then
		if [ ! -$opts "$target.$product_suffo" ]; then
			case "$dup" in
				mv)
					mv -f $target $target.$product_suffo || die "mv -f $target $target.$product_suff"
					;;
				cp)
					cp -fp $target $target.$product_suffo || die "cp -fp $target $target.$product_suff"
					;;
				*)
					p_echo " mk_backup: wrong option -- must be 'cp' or 'mv'"
					die "mk_backup"
					;;
			esac
		else
			case "$dup" in
				mv)
					mv -f $target $target.$product_suff || die "mv -f $target $target.$product_suff"
					;;
				cp)
					cp -fp $target $target.$product_suff || die "cp -fp $target $target.$product_suff"
					;;
				*)
					p_echo " mk_backup: wrong option -- must be 'cp' or 'mv'"
					die "mk_backup"
					;;
			esac
		fi
	else
		case "$opts" in
			f|d)
				;;
			*)
				p_echo " mk_backup: wrong option -- must be 'f' or 'd'"
				die "mk_backup"
				;;
		esac
	fi
}

# accumulates chown and chmod
set_ac()
{
	u_owner="$1"
	g_owner="$2"
	perms="$3"
	node="$4"

	chown $u_owner:$g_owner $node || die "chown $u_owner:$g_owner $node"
	chmod $perms $node || die "chmod $perms $node"
}

set_ac_r()
{
	u_owner="$1"
	g_owner="$2"
	perms="$3"
	node="$4"

	chown -R $u_owner:$g_owner $node || die "chown $u_owner:$g_owner $node"
	chmod -R $perms $node || die "chmod $perms $node"
}

add_manpath()
{
	local manpath manpath_config
	manpath="$1"

	if [ "X$manpath" = "X" ]; then
		warn "add_manpath: supplay manpath paramter"
		return 1
	fi


	case "$machine" in

		BSD*)
			manpath_config="/etc/manpath.config"

			if [ ! -f "$manpath_config" ]; then
				touch "$manpath_config"
			fi

			grep_res=`egrep "$manpath" "$manpath_config"`

			if [ "X$grep_res" != "X" ]; then
					return
			fi

			echo "OPTIONAL_MANPATH	$manpath" >> "$manpath_config"
			;;

		*)
			;;
	esac

	return 0
}

detect_vz()
{
	local vzcheck_file

	PLESK_VZ=0
	vzcheck_file="/proc/self/status"

	if [ ! -f ${vzcheck_file} ]; then
		return
	fi

	grep -q -E '^envID\:[[:space:]]*([[:digit:]]+)$' < ${vzcheck_file} >>"$product_log" 2>&1

	if [ $? -eq 0 ]; then
		PLESK_VZ=1
	fi
}

call_optional_function()
{
	local type_output="`type \"$1\" 2>/dev/null | head -n 1`"
	case "$type_output" in
		*function)
			"$@"
			;;
		*)
			return 0
			;;
	esac
}

# Super server internet library

super_server_type()
{
	case "$super_server" in

		'xinetd' | 'inetd')
			;;
		*)
			# check what super-server installed
			inten="define super-server internet daemon type"
			echo_try "$inten"
	
			if [ -f "$xinetd_conf" ]; then
				super_server="xinetd"
				xinetd_dir=`cat /etc/xinetd.conf |grep includedir| awk '{print $2}' -`
			elif [ -f "$inetd_conf" ]; then
				super_server="inetd"
			else
				die "$inten"
			fi
			p_echo "$super_server"
			;;
	esac
}

restart_super_server()
{
	if [ -n "$CHROOTISH" ]; then
			return 0
	fi

	super_server_type

	case "$super_server" in

		'xinetd')

		    inten="restart xinetd daemon"
		    echo_try "$inten"
		    case "$machine" in

 		        linux)
			    /etc/init.d/xinetd restart >> $product_log 2>&1
			    xinetd_pid=`$ps | grep xinetd | egrep "\?" | grep -v grep | head -1 | awk '{print $1}' -`
	            	    [ "X$xinetd_pid" != "X" ]  && suc || service_restart_warning "xinetd"
			    ;;
	 
		        *)
	            	    # not Linux
			    xinetd_pid=`$ps | grep xinetd | egrep "\?" | grep -v grep | head -1 | awk '{print $1}' -`
				if [ "X${xinetd_pid}" = "X" ]; then
#				die "${inten}: no running xinetd found"
					service_restart_warning "xinetd"
				else
				    $K_USR2 $xinetd_pid >> $product_log 2>&1 && suc || die "$inten"
				fi
	            	    ;;
								 
	            esac

		;;

		'inetd')

			inetd_pid=`$ps | grep inetd | egrep "\?" | grep -v grep | head -1 | awk '{print $1}' -`
			inten="restart inetd daemon"
			echo_try "$inten"

			if [ "X${inetd_pid}" = "X" ]; then
				 service_restart_warning "inetd"
			else
				$K_HUP $inetd_pid >> $product_log 2>&1 && suc || die "$inten"
			fi
		;;
	esac
}

comment_service_xinetd()
{
	#comment service records
	local dir file service grep_res
	service=$1

	inten="comment out record for service $service"
	echo_try "$inten"

	for dir in $xinetd_dir; do

		for file in $dir/*; do

			grep_res=`grep "$service" $file`
			case "$?" in
				0)
					awk "/^[[:space:]]*service[[:space:]]+$service($|[[:space:]]+)/,/.*}.*/ \
						{ print \"#\"\$0; next; }
						{print}
						" <$file >$file.tmp  &&
					mv -f $file.tmp $file || die "$inten" >> $product_log 2>&1
					restart_super_server
					;;
				*)
					;;
			esac

		done;

	done;

	suc

}

comment_service_inetd()
{
	local grep_res
	service=$1

	inten="comment out record for service $service"
	echo_try "$inten"

	grep_res=`grep "^$service[[:space:]]" $inetd_conf`

	case "$?" in
	
		0)
			sed -e "s|^$service|##commented_by_psa $service|g" < "$inetd_conf" > "$inetd_conf.${product}_tmp" \
			&& mv -f "$inetd_conf.${product}_tmp" "$inetd_conf" \
			&& suc || die >> $product_log 2>&1
			restart_super_server
			;;
		*)
			p_echo " no record for service $service found in the $inetd_conf"
			;;
	esac
}

comment_service()
{
	local service
	service=$1
	
	super_server_type

	case "$super_server" in

		'xinetd')
			comment_service_xinetd "$service"
			;;

		'inetd')
			comment_service_inetd "$service"
			;;

		*)
			die "Super server type is undefined"
			;;
	esac

}


remove_xinetd()
{
	local service file
	service=$1;
	
	rm -f $xinetd_dir/${service}_${product}
	
	#uncomment old records
	inten="comment out ${product} record for service $service"

	for file in $xinetd_dir/*; do
		awk "/^[[:space:]]*#[[:space:]]*service[[:space:]]+$service($|[[:space:]]+)/,/.*}.*/ \
			{ print substr(\$0,2); next; }
			{print}
			" <$file >$file.tmp &&
		mv -f $file.tmp $file >> $product_log 2>&1
	done;

	restart_super_server

}

remove_inetd()
{
	local service
	service=$1;

	case "$service" in

		ftp)
			inten="restore ftp service record in the $inetd_conf"

			sed -e "s|^ftp.*/${product}/ftpd/sbin/proftpd.*||g" \
				-e "s|ftp.*/${prev_product}/ftpd/sbin/proftpd.*||g" \
				< $inetd_conf > $inetd_conf.${product}_tmp \
			&& mv -f $inetd_conf.${product}_tmp $inetd_conf >> $product_log 2>&1
		    ;;

		smtp)
			inten="restore smtp service record in the $inetd_conf"

			sed -e "s|^smtp.*$QMAIL_ROOT_D/qmail/bin/tcp-env.*tcp-env.*/qmail/bin/relaylock.*||g" \
				-e "s|^smtp.*${product}/qmail/bin/tcp-env.*tcp-env.*/qmail/bin/relaylock.*||g" \
				-e "s|smtp.*${prev_product}/qmail/bin/tcp-env.*tcp-env.*/qmail/bin/relaylock.*||g" \
				< $inetd_conf > $inetd_conf.${product}_tmp \
			&& mv -f $inetd_conf.${product}_tmp $inetd_conf >> $product_log 2>&1
		    ;;

		smtps)
			inten="restore smtp service record in the $inetd_conf"

			sed -e "s|^smtps.*$QMAIL_ROOT_D/qmail/bin/tcp-env.*tcp-env.*/qmail/bin/relaylock.*||g" \
				-e "s|^smtps.*${product}/qmail/bin/tcp-env.*tcp-env.*/qmail/bin/relaylock.*||g" \
				-e "s|smtps.*${prev_product}/qmail/bin/tcp-env.*tcp-env.*/qmail/bin/relaylock.*||g" \
				< $inetd_conf > $inetd_conf.${product}_tmp \
			&& mv -f $inetd_conf.${product}_tmp $inetd_conf >> $product_log 2>&1
		    ;;

		pop3)
			inten="restore pop3 service record in the $inetd_conf"

			sed -e "s|^pop[-]*3.*/qmail/bin/tcp-env.*tcp-env.*/qmail/bin/pop3$||g" \
				-e "s|^pop[-]*3.*/qmail/bin/tcp-env.*tcp-env.*/qmail/bin/pop3$||g" \
				< $inetd_conf > $inetd_conf.${product}_tmp \
			&& mv -f $inetd_conf.${product}_tmp $inetd_conf >> $product_log 2>&1
			
		    ;;
		poppassd)
			# poppassd stream tcp nowait root /usr/sbin/tcpd /usr/local/psa/admin/bin/poppassd
			inten="restore poppassd service record in the $inetd_conf"

			sed -e "s|^poppassd.*$PRODUCT_ROOT_D/admin/bin/poppassd.*||g" \
				< $inetd_conf > $inetd_conf.${product}_tmp \
			&& mv -f $inetd_conf.${product}_tmp $inetd_conf >> $product_log 2>&1
			;;
		
	esac

	# uncomment old records
	sed -e "s|^##commented_by_psa $service|$service|g" < $inetd_conf > $inetd_conf.${product}_tmp \
	&& mv -f $inetd_conf.${product}_tmp $inetd_conf >> $product_log 2>&1

	restart_super_server

}

remove_super_server()
{
	local service
	service=$1;

	super_server_type

	case "$super_server" in

		xinetd)
			remove_xinetd "$service"
			;;
		inetd)
			remove_inetd "$service"
			;;
	esac
}

setup_xinetd()
{
	local service file rbl_on rbl_server
	service=$1;
	rbl_on=$2;
	rbl_server=$3;
	
	#comment old records
	inten="comment out old record for service $service"
	echo_try "$inten"

	for file in $xinetd_dir/*; do
		egrep '^[[:space:]]*service[[:space:]]+$service($|[[:space:]]+)' $file  > /dev/null
		[ $? -ne 0 ] && continue
		awk "/^[[:space:]]*service[[:space:]]+$service($|[[:space:]]+)/,/.*}.*/ \
			{ print \"#\"\$0; next; }
			{print}
			" <$file >$file.tmp  &&
		mv -f $file.tmp $file || die "$inten" >> $product_log 2>&1
	done;
	suc

	#setup
	service=$1;
	inten="set up record for service $service"
	echo_try "$inten"

	case "$service" in

		ftp)
			echo "
service ftp
{
	socket_type		= stream
	protocol		= tcp
	wait			= no
	disable			= no
	user			= root
	instances		= UNLIMITED
	server			= $PRODUCT_ROOT_D/ftpd/sbin/proftpd
	server_args		= -c $PRODUCT_ROOT_D/ftpd/etc/proftpd.conf
}" > "$xinetd_dir/ftp_${product}" || die "$inten"
			;;

		smtp)

			case "$rbl_on" in

				on)				
					echo "
service smtp
{
	socket_type     = stream
	protocol        = tcp
	wait            = no
	disable		= no
	user            = root
	instances       = UNLIMITED
	server          = $QMAIL_ROOT_D/bin/tcp-env
	server_args     = $RBLSMTPD $rbl_server $QMAIL_ROOT_D/bin/relaylock $QMAIL_ROOT_D/bin/qmail-smtpd $QMAIL_ROOT_D/bin/smtp_auth $TRUE_BIN $QMAIL_ROOT_D/bin/cmd5checkpw $TRUE_BIN
}" > "$xinetd_dir/smtp_${product}" || die "$inten"
					;;

				*)
					echo "
service smtp
{
	socket_type     = stream
	protocol        = tcp
	wait            = no
	disable		= no
	user            = root
	instances       = UNLIMITED
	server          = $QMAIL_ROOT_D/bin/tcp-env
	server_args     = $QMAIL_ROOT_D/bin/relaylock $QMAIL_ROOT_D/bin/qmail-smtpd $QMAIL_ROOT_D/bin/smtp_auth $TRUE_BIN $QMAIL_ROOT_D/bin/cmd5checkpw $TRUE_BIN
}" > "$xinetd_dir/smtp_${product}" || die "$inten"
					;;
			esac	
			;;

		smtps)
			case "$rbl_on" in
				on)				
					echo "
service smtps
{
	socket_type     = stream
	protocol        = tcp
	wait            = no
	disable		= no
	user            = root
	instances       = UNLIMITED
	server          = $QMAIL_ROOT_D/bin/tcp-env
	server_args     = $RBLSMTPD $rbl_server $QMAIL_ROOT_D/bin/relaylock $QMAIL_ROOT_D/bin/qmail-smtpd $QMAIL_ROOT_D/bin/smtp_auth $TRUE_BIN $QMAIL_ROOT_D/bin/cmd5checkpw $TRUE_BIN
}" > "$xinetd_dir/smtps_${product}" || die "$inten"
					;;
				*)
					echo "
service smtps
{
	socket_type     = stream
	protocol        = tcp
	wait            = no
	disable		= no
	user            = root
	instances       = UNLIMITED
	server          = $QMAIL_ROOT_D/bin/tcp-env
	server_args     = $QMAIL_ROOT_D/bin/relaylock $QMAIL_ROOT_D/bin/qmail-smtpd $QMAIL_ROOT_D/bin/smtp_auth $TRUE_BIN $QMAIL_ROOT_D/bin/cmd5checkpw $TRUE_BIN
}" > "$xinetd_dir/smtps_${product}" || die "$inten"
					;;
			esac	
			;;

		poppassd)
			echo "
service poppassd
{
	socket_type		= stream
	protocol		= tcp
	port			= 106
	wait			= no
	disable			= no
	user			= root
	instances		= 1000
	flags			= KEEPALIVE
	server			= $PRODUCT_ROOT_D/admin/bin/poppassd
}" > "$xinetd_dir/poppassd_${product}" || die "$inten"
			;;

	esac

	suc

}

setup_inetd()
{
	local service rbl_on rbl_server

	service=$1;
	rbl_on=$2;
	rbl_server=$3;
	
	local ftp_rec smtp_rec pop3_rec 

	ftp_rec="ftp stream tcp nowait root $PRODUCT_ROOT_D/ftpd/sbin/proftpd -c $PRODUCT_ROOT_D/ftpd/etc/proftpd.conf"

	case "$rbl_on" in
		on)
			smtp_rec="smtp stream tcp nowait root $QMAIL_ROOT_D/bin/tcp-env tcp-env  $RBLSMTPD $rbl_server $QMAIL_ROOT_D/bin/relaylock $QMAIL_ROOT_D/bin/qmail-smtpd $QMAIL_ROOT_D/bin/smtp_auth $QMAIL_ROOT_D/bin/true $QMAIL_ROOT_D/bin/cmd5checkpw $QMAIL_ROOT_D/bin/true"
			smtps_rec="smtps stream tcp nowait root $QMAIL_ROOT_D/bin/tcp-env tcp-env  $RBLSMTPD $rbl_server $QMAIL_ROOT_D/bin/relaylock $QMAIL_ROOT_D/bin/qmail-smtpd $QMAIL_ROOT_D/bin/smtp_auth $QMAIL_ROOT_D/bin/true $QMAIL_ROOT_D/bin/cmd5checkpw $QMAIL_ROOT_D/bin/true"
			;;
		*)
			smtp_rec="smtp stream tcp nowait root $QMAIL_ROOT_D/bin/tcp-env tcp-env  $QMAIL_ROOT_D/bin/relaylock $QMAIL_ROOT_D/bin/qmail-smtpd $QMAIL_ROOT_D/bin/smtp_auth $QMAIL_ROOT_D/bin/true $QMAIL_ROOT_D/bin/cmd5checkpw $QMAIL_ROOT_D/bin/true"
			smtps_rec="smtps stream tcp nowait root $QMAIL_ROOT_D/bin/tcp-env tcp-env  $QMAIL_ROOT_D/bin/relaylock $QMAIL_ROOT_D/bin/qmail-smtpd $QMAIL_ROOT_D/bin/smtp_auth $QMAIL_ROOT_D/bin/true $QMAIL_ROOT_D/bin/cmd5checkpw $QMAIL_ROOT_D/bin/true"
			;;
	esac

	case "$machine" in
		BSD*)
			poppassd_rec="poppassd stream tcp nowait/1000 root /usr/libexec/tcpd $PRODUCT_ROOT_D/admin/bin/poppassd"
			;;
		*)
			poppassd_rec="poppassd stream tcp nowait/1000 root /usr/sbin/tcpd $PRODUCT_ROOT_D/admin/bin/poppassd"
			;;
	esac

	case "$service" in

		ftp)
			if [ -x /usr/sbin/update-inetd ]; then

				# Use Debian inetd config manipulation facility
				/usr/sbin/update-inetd --add "$ftp_rec"

			else
				grep_res=`egrep "^ftp.*/${product}/ftpd/sbin/proftpd*." $inetd_conf`
				inten="add ftp service (proftpd) record into the $inetd_conf file"
				echo_try $inten

				case "$grep_res" in
					ftp*)
						p_echo
						p_echo " ftp service (proftpd) is already in the $inetd_conf"
						;;
					*)
						comment_service $service
						echo "$ftp_rec" >> $inetd_conf && suc || die
						;;
				esac
			fi
			;;

		smtp)
			if [ -x /usr/sbin/update-inetd ]; then

				# Use Debian inetd config manipulation facility
				/usr/sbin/update-inetd --add "$smtp_rec"

			else
				egrep "^smtp.*$QMAIL_ROOT_D/bin/tcp-env.*tcp-env.*$QMAIL_ROOT_D/bin/relaylock.*$" "$inetd_conf" >> $product_log 2>&1
				case "$?" in
					0)
						sed -e "/^smtp/d" < "$inetd_conf" > "$inetd_conf.${product}_tmp" \
						&& mv -f "$inetd_conf.${product}_tmp" "$inetd_conf" || die >> $product_log 2>&1
						;;
					1)
						comment_service $service
						;;
					*)
						err
						;;
				esac

				inten="add smtp service record into the $inetd_conf file"
				echo_try $inten
				echo "$smtp_rec"  >> $inetd_conf  && suc || die $inten
			fi
			;;

		smtps)

			if [ -x /usr/sbin/update-inetd ]; then

				# Use Debian inetd config manipulation facility
				/usr/sbin/update-inetd --add "$smtps_rec"

			else

				egrep "^smtps.*$QMAIL_ROOT_D/bin/tcp-env.*tcp-env.*$QMAIL_ROOT_D/bin/relaylock.*$" "$inetd_conf" >> $product_log 2>&1
				case "$?" in
					0)
						sed -e "/^smtps/d" < "$inetd_conf" > "$inetd_conf.${product}_tmp" \
						&& mv -f "$inetd_conf.${product}_tmp" "$inetd_conf" || die >> $product_log 2>&1
						;;
					1)
						comment_service $service
						;;
					*)
						err
						;;
				esac

				inten="add smtps service record into the $inetd_conf file"
				echo_try $inten
				echo "$smtps_rec"  >> $inetd_conf  && suc || die $inten
			fi
			;;

		poppassd)

			if [ -x /usr/sbin/update-inetd ]; then

				# Use Debian inetd config manipulation facility
				/usr/sbin/update-inetd --add "$poppassd_rec"
			else
				#	^poppassd.*$PRODUCT_ROOT_D/admin/bin/poppassd.*
				grep_res=`egrep "^poppassd.*$PRODUCT_ROOT_D/admin/bin/poppassd.*" $inetd_conf`
				inten="add poppassd service record in the $inetd_conf"
				echo_try $inten
		
				case "$grep_res" in
					poppassd*)
						p_echo
						p_echo " poppassd service is already in the $inetd_conf"
						;;
					*)
						comment_service $service
						echo "$poppassd_rec" >> $inetd_conf && suc || die
						;;
				esac
			fi
			;;
	esac

}

setup_super_server()
{
	local service rbl_on rbl_server
	service=$1;
	rbl_on=$2;
	rbl_server=$3;

	super_server_type

	case "$super_server" in

		xinetd)
			setup_xinetd "$service" "$rbl_on" "$rbl_server"
			;;
		inetd)
			setup_inetd "$service" "$rbl_on" "$rbl_server"
			;;
	esac

	restart_super_server
}


add_service2etc_services()
{
	### this function adds service description into /etc/services

	service=$1
	port=$2
	protocol=$3 ### can be empty

	if [ -z "$service" ]; then
		warn "add_service_network_service() error: service does not specified."
		return
	fi

	echo $port | grep '^[0-9][0-9]*$' >/dev/null
	if [ 0$? -ne 0 ]; then
		warn "add_service_network_service() error: '$service' service port number '$port' is invalid."
		return
	fi

	if [ ! -z "$protocol" ]; then
		if [ "$protocol" != "tcp" -a  "$protocol" != "udp" ]; then
			warn "add_service_network_service() error: '$service' service protocol name '$protocol' is invalid."
			return
		fi
		protocol_expr="$protocol"
	else
		protocol_expr='..*'
	fi

	grep "^[[:space:]]*$service[[:space:]]*$port\/$protocol_expr"'$' $services >/dev/null
	if [ $? -ne 0 ]; then
		awk '{if ($1 != "'$service'" && $2 ~ "^'$port'\/'$protocol_expr'([^0-9]|$)") {print "#"$0"\n'$service'\t"$2} else {print $0}}' \
		<$services >"$services.tmp" \
		&& mv -f "$services.tmp" $services >> $product_log 2>&1
	fi
}

get_pid()
{
	local i

	ex_f="$1"
	opt="$2"
	owner="$3"

	min_num=`expr 1`

	case "$opt" in
		false)
			for i in `$ps_long | grep $ex_f | grep -v grep | grep -v httpsdctl | grep -v apachectl | awk '{print $2}' -`; do
				min_num=$i
				break
			done
			;;
		true)
			for i in `$ps_long | grep $ex_f | grep -v grep | grep -v httpsdctl | grep -v apachectl | grep "$owner" | awk '{print $2}' -`; do
				min_num=$i
				break
			done
			;;
		*)
			p_echo "get_pid: wrong parameter"
			die "get_pid $ex_f $opt $owner"
			;;
	esac

	common_var=$min_num
	return $min_num
}

kill_pids()
{
	ex_f="$1"
	owner="$2"

	for i in `$ps_long | grep $ex_f | grep -v grep | grep -v httpsdctl | grep -v apachectl | grep $owner | awk '{print $2}' -`; do
		if [ $i -gt 1 ]; then
			$K_TERM $i >> $product_log 2>&1
		fi
	done
}

delete_user()
{
	local rm_user

	rm_user="$1"

	# if it is mailman user, remove its crontab from system
	if [ "X${rm_user}" = "X${mailman_user}" ]; then

		inten="remove crontab of ${rm_user}"
		echo "y" | $crontab -u "${mailman_user}" -r >> $product_log 2>&1 || die "$inten"

	fi

	inten="remove user $rm_user"
	echo_try "$inten"

	case "$machine" in
		BSD)
			echo "y" | pw userdel $rm_user>> $product_log 2>&1 && suc || die "$inten"
			pwd_mkdb /etc/master.passwd >> $product_log 2>&1
		;;
		BSDI)
			rmuser $rm_user>> $product_log 2>&1 && suc || die "$inten"
			pwd_mkdb /etc/master.passwd >> $product_log 2>&1

		;;
		linux|solaris)
			userdel  $rm_user>> $product_log 2>&1 && suc || die "$inten"
		;;
	esac
}

delete_group()
{
	local rm_group

	rm_group="$1"

	inten="remove group $rm_group"
	echo_try "$inten"

	case "$machine" in
		BSD)
			pw groupdel $rm_group>> $product_log 2>&1
			pwd_mkdb /etc/master.passwd >> $product_log 2>&1
		;;
		BSDI)
			rmgroup $rm_group>> $product_log 2>&1
			pwd_mkdb /etc/master.passwd >> $product_log 2>&1
		;;
		linux|solaris)
			mk_backup "/etc/group" cp f
			if [ -f "/etc/group" ]; then
				sed -e "/$rm_group/d" < /etc/group > /etc/group.tmp || die $inten
				mv -f /etc/group.tmp /etc/group  >> $product_log 2>&1
				if [ "$?" -ne 0 ]; then
					rsr_backup "/etc/group" cp f
					die $inten
				fi
			fi
#		groupdel  $rm_group>> $product_log 2>&1
		;;
	esac
	suc
}

#set_params

set_common_params()
{

	PATH=/sbin:/bin:/usr/sbin:/usr/bin:/usr/local/sbin:/usr/local/bin
	LANG=C
	export PATH LANG
	umask 022
	ulimit -n 65535

	K_HUP="/bin/kill -HUP"
	K_KILL="/bin/kill -KILL"
	K_TERM="/bin/kill -TERM"
	K_USR2="/bin/kill -USR2"

	users_created=""
	groups_created=""

	certificate_file="$PRODUCT_ROOT_D/etc/httpsd.pem"
	product=${PRODNAME}
	START_SH="$PRODUCT_RC_D/${product}"

	clients_group="${product}cln"
	clients_GID=10001

	services_group="${product}serv"
	services_GID=10003

	pop_user="popuser"
	pop_UID=110
	pop_group="popuser"
	pop_GID=110

	product_suff="saved_by_${product}".`date "+%m.%d;%H:%M"`
	product_suffo="saved_by_${product}"

	common_var=0

	services="/etc/services"
	fstab="/etc/fstab"
	mtab="/etc/mtab"
	xinetd_conf="/etc/xinetd.conf"
	xinetd_dir="/etc/xinetd.d"
	inetd_conf="/etc/inetd.conf"
	syslog_conf="/etc/syslog.conf"
	get_hostname="hostname"
	get_domainname="domainname"

	#VZP used to determine that we're inside SVE
	vza_file="/var/vzagent"

	PREV_PRODUCT_ROOT_D="/usr/local/${prev_product}"

	#default parameters
	tar="tar"
	crontab="/usr/bin/crontab"

	#machine depended parameters
	TARGET_OS="`uname -a`"
	case "$TARGET_OS" in

		BSD/OS*BSDI*i386)
			machine="BSDI"
			sendmail="/usr/sbin/sendmail"
			mail_local="/usr/libexec/mail.local"
			ps="ps axw"
			ps_long="ps axwu"
			dummy_shell="/sbin/nologin"
			dummy_home="/"
			false_shell="/usr/bin/false"
			compress="compress"
			uncompress="uncompress -c"
			uudecode="uudecode -o /dev/stdout"
			uudecode_full="/usr/bin/uudecode"
			ifconfig="/sbin/ifconfig -a"
			inet_str="inet"
			named_osrelease=0
			;;

		FreeBSD*i386)
			machine="BSD"
			sendmail="/usr/sbin/sendmail"
			mail_local="/usr/libexec/mail.local"
			ps="ps axw"
			ps_long="ps axfwu"
			dummy_shell="/sbin/nologin"
			dummy_home="/"
			false_shell="/usr/bin/false"
			bash_shell="/usr/local/bin/bash"
			rbash_shell="/usr/local/bin/rbash"
			compress="gzip -9 -c"
			uncompress="gunzip -c"
			uudecode="uudecode -p"
			uudecode_full="/usr/bin/uudecode"
			ifconfig="/sbin/ifconfig -a"
			inet_str="inet"
			os_version=`uname -v | sed -e 's/^.*FreeBSD[ \t][ \t]*\([0-9\.][0-9\.]*\)-.*$/\1/'`
			if tar --version | grep bsdtar; then
				echo "Use gtar"
				which gtar && tar="gtar" || exit 1
			fi
			named_osrelease=040100 # "must be >= 020218 for  named -u <named> "
			;;

		SunOS*5.*sun4*)
			machine="solaris"
			inetd_conf="/etc/inet/inetd.conf"
			services="/etc/inet/services"
			crontab="/bin/crontab"
			fdtl=""
			sendmail="/usr/lib/sendmail"
			sndml_ini="/etc/init.d/sendmail"
			mail_local="/usr/lib/mail.local"
			ps="ps -e"
			ps_long="ps -aef"
			dummy_shell="/bin/false"
			dummy_home="/"
			false_shell="/bin/false"
			compress="compress"
			uncompress="uncompress"
			uudecode="uudecode -p"
			uudecode_full="/bin/uudecode"
			tar="gtar"
			ifconfig="/usr/sbin/ifconfig -a"
			inet_str="inet"
			named_osrelease=0
			;;

		Linux*2.?.*i?86*|Linux*2.?.*x86_64*)
			get_hostname="hostname -f"
			if [ -f /etc/slackware-version ]; then
			    machine="linux"
			    linux_distr="slackware"
			    fdtl=""
			    sendmail="/usr/sbin/sendmail"
			    sndml_ini="/etc/rc.d/init.d/sendmail"
			    mail_local="/usr/libexec/mail.local"
			    ps="ps axw"
			    ps_long="ps axuw"
			    dummy_shell=""
			    dummy_home="/"
			    false_shell="/bin/false"
			    compress="gzip -9 -c"
			    uncompress="gunzip -c"
			    uudecode="uudecode -o /dev/stdout"
			    ifconfig="/sbin/ifconfig -a"
			    inet_str="inet addr"
			    named_osrelease=0
			else
			    machine="linux"
				if [ -f /etc/mandrake-release ]; then
			    	linux_distr="mandrake"
				elif [ -f /etc/fedora-release ]; then
			    	linux_distr="fedora"
				elif [ -f /etc/SuSE-release ]; then
			    	linux_distr="suse"
				elif [ -f /etc/debian_version ]; then
			    	linux_distr="debian"
					get_domainname="dnsdomainname"
				else
			    	linux_distr="redhat"
				fi
			    fdtl="-M"
			    sendmail="/usr/sbin/sendmail"
			    sndml_ini="/etc/init.d/sendmail"
			    mail_local="/usr/libexec/mail.local"
			    ps="ps axw"
			    ps_long="ps axuw"
			    dummy_shell="/bin/false"
			    dummy_home="/"
			    false_shell="/bin/false"
				bash_shell="/bin/bash"
				rbash_shell="/bin/rbash"
			    compress="gzip -9 -c"
			    uncompress="gunzip -c"
			    uudecode="uudecode -o /dev/stdout"
			    uudecode_full="/usr/bin/uudecode"
			    ifconfig="/sbin/ifconfig -a"
			    inet_str="inet addr"
			    named_osrelease=`cat /proc/sys/kernel/osrelease | perl -F"/[.-]/" -n -a  -e 'printf "%02u%02u%02u\n", $F[0],$F[1],$F[2]'`
			fi
			;;
		*)
			echo
			echo "Sorry, but your configuration isn't supported yet"
			echo "uname is $TARGET_OS"
			echo
			echo "For more information, please contact $support_email"
			echo
			;;
	esac

	# setup echo
	case "`echo 'x\c'`" in
		'x\c')
			echo="echo -n"
			nnl=
			;;
		x)
			echo="echo"
			nnl="\c"
			;;
		*)
			echo "ERROR while trying to set up echo"
			exit 1
			;;
	esac

	# setup process
	if [ "X$process" = "X" ]; then
		if   [ "X$do_reconfigure" = "X1" ]; then
	    	process="reconfiguration"
		elif  [ "X$do_upgrade" = "X1" ]; then
	    	process="upgrade"
		else
		    process="installing"
		fi
	fi

#	common_var="real"
#	if [ -f $PRODUCT_ROOT_D/version -a "X$do_patch" = "X" ]; then
#	    common_var=`cat $PRODUCT_ROOT_D/version | tr '[:blank:]' '_'`
	product_log="/tmp/${product}_${product_version}_${conceived_os_vendor}_${conceived_os_version}_build${product_build}_${process}.log"
	product_problems_log="/tmp/${product}_${product_version}_${conceived_os_vendor}_${conceived_os_version}_build${product_build}_problems.log"
#	elif [ "X$my_name_1" != "X" ]; then
#	    product_log="/tmp/${my_name_1}.log"
#	else
#	    product_log="/tmp/${product}_default_${process}.log"
#	fi

	detect_vz
}
# Warning! use this function only for Linux not for FreeBSD
# This function tries to register service in different ways
# call register_service qmail levels_start levels_stop alt_link alt_name alt_path alt_priority alt_initscript
# Only first parameter is necessessary

register_service(){
	local name=$1
	local levels_start=$2
	local levels_stop=$3

	local updatercd="/usr/sbin/update-rc.d"
	local chkconfig="/sbin/chkconfig"
	local insserv="/sbin/insserv"
		
	if [ -z "$name" ]; then
		return 0
	fi
	
	local inten="register service $name"
	echo_try $inten
	
	if [ -x "$updatercd" ]; then
		p_echo " using $updatercd"
		if [ "X$levels_start" != "Xdefaults" -a -n "$levels_start" ]; then
			start="start 20 $levels_start ."
		fi
			
		if [ "X$levels_stop" != "Xdefaults" -a -n "$levels_stop" ]; then
			stop="stop 20 $levels_stop ."
		fi

		if [ -z "$start$stop" ]; then
			$updatercd $name defaults 20 || warn $inten 
		else
			$updatercd $name $start $stop || warn $inten
		fi
			
	elif [ -x "$insserv" ]; then
		p_echo " using $insserv"	
		$insserv -f $name || warn $inten
	elif [ -x "$chkconfig" ]; then
		p_echo " using $chkconfig"	
		$chkconfig --add $name || warn $inten
	
		if [ "X$levels_start" != "Xdefaults" -o -z "$levels_start" ]; then
			$chkconfig --level `echo $levels_start | tr -d ' '` $name on || warn $inten
		fi
				
		if [ "X$levels_start" != "Xdefaults" -o -z "$levels_stop" ]; then
			$chkconfig --level `echo $levels_stop | tr -d ' '` $name off || warn $inten
		fi
	else 
		die "Unable to find suitable application for register: $updatercd, $chkconfig or $insserv"
	fi
	
	suc	
	if [ -x "/usr/sbin/alternatives" ]; then
		inten="register $name in alternatives"
		echo_try $inten
		if [ ! -z "$8" ]; then
			/usr/sbin/alternatives --install $4 $5 $6 $7 --initscript $8 || warn $inten
		elif [ ! -z "$7" ]; then
		 	/usr/sbin/alternatives --install $4 $5 $6 $7 || warn $inten
		fi
		suc
	fi
}
# Unregister service 
# call unregister_service $name stop remove
# if $2 = stop then we try to stop service before unregister
# if $3 = remove then we remove start script 
unregister_service(){
	local name=$1
	local updatercd="/usr/sbin/update-rc.d"
	local chkconfig="/sbin/chkconfig"
	local insserv="/sbin/insserv"
	
	if [ "X$1" = "X" ]; then
		die "unregister_service: you must specify at least name of service!"
	fi
	
	local inten="unregister service $name"
	echo_try $inten
	
	if [ "X$2" = "Xstop" -a -x /etc/init.d/$name ]; then
		/etc/init.d/$name stop
	fi
	
	if [ -x "$updatercd" ]; then
		p_echo " using $updatercd"
		$updatercd -f $name	remove || warn "Unable to unregister $name using $updatercd"
	elif [ -x $insserv ]; then
		p_echo " using $insserv"
		$insserv -r -f $name || warn "Unable to unregister $name using $insserv"
	elif [ -x "$chkconfig" ]; then
		p_echo " using $chkconfig"	
		$chkconfig --del $name ||  warn "Unable to unregister $name using $chkconfig"
	else
		die "Unable to find suitable application for register: $updatercd, $chkconfig or $insserv"
	fi		
	if [ "X$3" = "Xremove" -a -x /etc/init.d/$name ]; then
		rm -f /etc/init.d/$name
	fi
}

remove_modules()
{
	p_echo
	p_echo "===> Removing modules"

	local moduledir
	for moduledir in $PRODUCT_ROOT_D/var/modules/*; do
		if [ -d "$moduledir" -a -x "$moduledir/uninstall" ]; then
			local module=`basename $moduledist`
			echo_try "uninstall $module module"
			"$moduledir/uninstall" && suc || warn "uninstallation of $module module"
		fi
	done
}

# Invoke mysql
mysql()
{
	$mysql_client $mysql_auth $mysql_args -D$mysql_db_name "$@"
}

## @@constructor set_qmail_params

comment_out_comsat()
{

    if [ -f "$inetd_conf" ]; then
	mk_backup $inetd_conf cp f

	grep_res=`egrep '^comsat' $inetd_conf`
	inten="comment out comsat service record from $inetd_conf"
	echo_try $inten

	case "$grep_res" in
		comsat*)
			sed -e "s/^comsat/#comsat/g" < "$inetd_conf" > "$inetd_conf.${product}_tmp" || die
			mv -f "$inetd_conf.${product}_tmp" "$inetd_conf" && suc || die $inten
			;;
		*)
			p_echo " there is no comsat service record in $inetd_conf"
			;;
	esac
    fi
}

set_qmail_perms()
{
	inten="set up default qmail permissions"
	echo_try $inten

	set_ac 0 qmail 755 "$QMAIL_ROOT_D"
	set_ac 0 qmail 755 "$QMAIL_ROOT_D/*"
	set_ac 0 qmail 555 "$QMAIL_ROOT_D/bin/*"
	# rm drweb save of qmail-queue
#	if [ -f "$QMAIL_ROOT_D/bin/qmail-queue.orig" ]; then
#		rm -f "$QMAIL_ROOT_D/bin/qmail-queue.orig"
#	fi
	set_ac qmailq qmail 4511 "$QMAIL_ROOT_D/bin/qmail-queue"
	set_ac 0 qmail 555 "$QMAIL_ROOT_D/boot/*"

	set_ac alias qmail 2755 "$QMAIL_ROOT_D/alias"
	set_ac qmailq qmail 750 "$QMAIL_ROOT_D/queue"

	set_ac_r qmails qmail 700 "$QMAIL_ROOT_D/queue/bounce"
	set_ac_r qmails qmail 700 "$QMAIL_ROOT_D/queue/info"
	set_ac_r qmails qmail 700 "$QMAIL_ROOT_D/queue/local"
	set_ac_r qmails qmail 700 "$QMAIL_ROOT_D/queue/remote"

	set_ac_r qmailq qmail 700 "$QMAIL_ROOT_D/queue/intd"
	set_ac_r qmailq qmail 750 "$QMAIL_ROOT_D/queue/lock"
	set_ac_r qmailq qmail 750 "$QMAIL_ROOT_D/queue/mess"
	set_ac_r qmailq qmail 700 "$QMAIL_ROOT_D/queue/pid"
	set_ac_r qmailq qmail 750 "$QMAIL_ROOT_D/queue/todo"

	set_ac qmails qmail 600 "$QMAIL_ROOT_D/queue/lock/sendmutex"
	set_ac qmailr qmail 644 "$QMAIL_ROOT_D/queue/lock/tcpto"
	set_ac qmails qmail 622 "$QMAIL_ROOT_D/queue/lock/trigger"

	set_ac 0 0 555 "$QMAIL_ROOT_D/bin/relaylock"
	set_ac 0 0 4555 "$QMAIL_ROOT_D/bin/smtp_auth"
	set_ac 0 0 4555 "$QMAIL_ROOT_D/bin/cmd5checkpw"
	set_ac 0 qmail 4555 "$QMAIL_ROOT_D/bin/autoresponder"
	set_ac 0 0 4555 "$QMAIL_ROOT_D/plugins/chkrcptto"
	set_ac 0 $mm_mailgroup 2555 "$QMAIL_ROOT_D/bin/mm_wrapper"

	suc
}

qmail_start()
{
	if [ -n "$CHROOTISH" ]; then
			return 0
	fi
	inten="start Qmail services"
	echo_try "$inten"
	"$PRODUCT_RC_D/qmail" start >> $product_log 2>&1

	if [ $? -ne 0 ]; then

		if [ "X${service_failures_ignoring}" != "X1" ]; then
			die $inten
		else
			warn $inten
			minor_problem=1
		fi
	else
		suc
	fi

	sleep 4
}

qmail_stop()
{
	if [ -n "$CHROOTISH" ]; then
			return 0
	fi
	inten="stop Qmail services"
	echo_try "$inten"
	"$PRODUCT_RC_D/qmail" stop >> $product_log 2>&1 && suc || warn $inten
	sleep 4
}

set_maillog()
{
	inten="set maillog file to $PRODUCT_ROOT_D/var/log/maillog"
	echo_try "$inten"

	mail_log="$PRODUCT_ROOT_D/var/log/maillog"
	[ -d "$PRODUCT_ROOT_D/var/log" ] || mkdir -p "$PRODUCT_ROOT_D/var/log"
	touch $mail_log

	log_str="^mail.*[[:space:]]*$mail_log"
	grep "$log_str" $syslog_conf > /dev/null
	if [ "$?" -ne 0 ]; then
		grep '^mail\.\*' $syslog_conf > /dev/null
		if [ "$?" -eq 0 ]; then
			## if line "mail.*       ..." is exist then
			## replace this with new line
			sed -e "s|^mail\.\*.*$|mail.*						$mail_log|" \
			< $syslog_conf > $syslog_conf.tmp && \
			mv -f $syslog_conf.tmp $syslog_conf || die "$inten"
		else
			## if line "mail.*       ..." is NOT exist then
			## search "*.       ..." line
			num=`awk '{if ((my==0) && (index($1, "*.") == 1)) {my=1; print FNR;}}' < $syslog_conf`
			if [ "0$num" -gt "0" ]; then
				## if line "*.       ..." is exist then
				## disable all lines beginning with "mail."
				## and insert new line "mail.*      ..." before this
				sed -e 's/^\(mail\.\)/#\1/' \
					-e ''${num}'i\
					mail.*						'$mail_log'' \
					< $syslog_conf > $syslog_conf.tmp && \
				mv -f $syslog_conf.tmp $syslog_conf || die "$inten"
			else
				## if line "*.       ..." is NOT exist then
				## disable all lines beginning with "mail."
				## and insert new line "mail.*      ..." at the end of file
				sed -e 's/^\(mail\.\)/#\1/'	< $syslog_conf > $syslog_conf.tmp && \
				echo "mail.*						$mail_log" >> $syslog_conf.tmp && \
				mv -f $syslog_conf.tmp $syslog_conf || die "$inten"
			fi
		fi
	fi

	# restart syslogd
	syslogd_pid=`ps ax | grep syslog | egrep "\?" | grep -v grep | head -1 | awk '{print $1}' -`
	if [ "X${syslogd_pid}" != "X" -a -z "$CHROOTISH" ]; then
		kill -HUP $syslogd_pid >> $product_log 2>&1
	fi

	suc
}

fix_mailman_cfg()
{
	inten="fix mailman configuration"
	# when installing psa, psa.conf does not exists
	# so we're to test all possible places
	for MAILMAN_ROOT_D in /var/mailman /usr/lib/mailman; do
		mm_cfg="$MAILMAN_ROOT_D/Mailman/mm_cfg.py"
		if test -r "$mm_cfg" && grep -q "^MTA[[:space:]]*=" "$mm_cfg"; then
			echo_try "$inten in $MAILMAN_ROOT_D"
			grep -v "^MTA[[:space:]]*=" "$mm_cfg" > "$mm_cfg.new" || die "$inten"
			mv -f "$mm_cfg.new" "$mm_cfg" || die "$inten"
			suc
		fi
	done
}

install_qmail()
{
	if [ "X$DEMO_VERSION" = "Xyes" ]; then
		return 0;
	fi
	p_echo
	p_echo "===> Installing Qmail server"

	set_maillog

	p_echo " Configuring Qmail..."

	# At upgrade time, old qmail tunings keeped untouched
	if [ ! -f "$QMAIL_ROOT_D/control/me" ]; then
		echo "$fullhost"	> "$QMAIL_ROOT_D/control/me" || die
	fi

	if [ ! -f "$QMAIL_ROOT_D/control/locals" ]; then
		echo "localhost"	> "$QMAIL_ROOT_D/control/locals" || die
	fi

	touch "$QMAIL_ROOT_D/control/rcpthosts" || die
	touch "$QMAIL_ROOT_D/control/virtualdomains" || die
	touch "$QMAIL_ROOT_D/users/poppasswd" || die

	chown 0:0 "$QMAIL_ROOT_D/control"/* || die
	chmod 644 "$QMAIL_ROOT_D/control"/* || die
	chmod 600 "$QMAIL_ROOT_D/users/poppasswd" || die


#	( echo "$fullhost" | sed 's/^\([^\.]*\)\.\([^\.]*\)\./\2\./' | (
#	read ddom
#  	inten="put $ddom into $QMAIL_ROOT_D/control/defaultdomain"
#	echo_try $inten
#	echo "$ddom" > $QMAIL_ROOT_D/control/defaultdomain &&
#	suc || die $inten
#	) )

#	( echo "$fullhost" | sed 's/^.*\.\([^\.]*\)\.\([^\.]*\)$/\1.\2/' | (
#	read pdom
#	inten="put $pdom into $QMAIL_ROOT_D/control/plusdomain"
#	echo_try $inten
#	echo "$pdom" > $QMAIL_ROOT_D/control/plusdomain &&
#	suc || die $inten
#	) )

#	inten="put $fqdn into $QMAIL_ROOT_D/control/locals"
#	echo_try $inten
#	echo "$fullhost" >> $QMAIL_ROOT_D/control/locals &&
#	suc || die $inten

#	inten="put $fullhost into $QMAIL_ROOT_D/control/rcpthosts"
#	echo_try $inten
#	echo "$fullhost" >> $QMAIL_ROOT_D/control/rcpthosts &&
#	suc || die $inten
#	echo "Now qmail will refuse to accept SMTP messages except to $fullhost"
#	echo "Make sure to change rcpthosts if you add hosts to locals or virtualdomains!"

	#echo "&$admin_email" > $QMAIL_ROOT_D/alias/.qmail-postmaster || die
	#echo "&$admin_email" > $QMAIL_ROOT_D/alias/.qmail-mailer-daemon || die
	#echo "&$admin_email" > $QMAIL_ROOT_D/alias/.qmail-root || die

#	set_ac root 0 644 $QMAIL_ROOT_D/alias/.qmail* || die


	if [ "X$do_upgrade" != "X1" ]; then
		remove_sendmail

		[ -x "/etc/init.d/postfix" ] && unregister_service postfix

		comment_out_comsat

		setup_super_server "smtp"

		if [ "$machine" = "linux" -a "$linux_distr" = "suse" ]; then
			add_service2etc_services 'smtps' 465 'tcp'
		fi
		setup_super_server "smtps"
	fi

	generate_tls_certificates
	[ -x "/etc/init.d/qmail" ] && register_service qmail defaults defaults /usr/sbin/sendmail mta /var/qmail/bin/sendmail 90
	
	qmail_start

	if [ "X$do_upgrade" != "X1" ]; then
		qmail_stop
	fi

	if [ ! -d "$QMAIL_ROOT_D/mailnames" ]; then
		inten="make $QMAIL_ROOT_D/mailnames directory"
		echo_try $inten
		mkdir "$QMAIL_ROOT_D/mailnames" && suc || die $inten
		chown root:qmail "$QMAIL_ROOT_D/mailnames" ||
		die "chown root:qmail $QMAIL_ROOT_D/mailnames"
	fi

	if [ ! -f "$QMAIL_ROOT_D/users/assign" ]; then
		echo "." > "$QMAIL_ROOT_D/users/assign" || err
        	set_ac 0 0 644 "$QMAIL_ROOT_D/users/assign" || err
	fi

	ln -fs "$QMAIL_ROOT_D/bin/sendmail" "$sendmail" ||
		die "ln -fs $QMAIL_ROOT_D/bin/sendmail $sendmail"

	ln -fs "$QMAIL_ROOT_D/bin/sendmail" "/usr/lib/sendmail" ||
		die "ln -fs $QMAIL_ROOT_D/bin/sendmail /usr/lib/sendmail"

	# for mdk, must be on debian too
	if type update-alternatives >/dev/null 2>&1; then
		update-alternatives --install /usr/sbin/sendmail mta "$QMAIL_ROOT_D/bin/sendmail" 30 \
			    --slave /usr/lib/sendmail mta-in_libdir "$QMAIL_ROOT_D/bin/sendmail"
	fi

	# for many
	fix_mailman_cfg

	add_manpath "$QMAIL_ROOT_D/man"


###  FOR UPGRADER! For each mailname:

	#if [ "X$do_upgrade" = "X1" ]; then

#		for dom_name in `$mysql <<END
#select name from domains;
#END
#`	; do
#			for mail_name in `$mysql <<END
#select m.mail_name from mail m, domains d where m.dom_id=d.id and d.name='$dom_name' ;
#END
#`			; do
#				mkdir $QMAIL_ROOT_D/mailnames/$dom_name/$mail_name/@attachments  >> $product_log 2>&1
#				chown popuser:popuser $QMAIL_ROOT_D/mailnames/$dom_name/$mail_name/@attachments  >> $product_log 2>&1
#				chmod 700 $QMAIL_ROOT_D/mailnames/$dom_name/$mail_name/@attachments  >> $product_log 2>&1
#			done
#		done

	#fi

}

# $1 - openssl binary
# $2 - openssl args
# $3 - cert file name
# $4 - bits number
qmail_tls_cert()
{
	local QMAIL_CONTROL_D TLS_CERT

	QMAIL_CONTROL_D=${QMAIL_ROOT_D}/control
	TLS_CERT=${QMAIL_CONTROL_D}/$3

	if [ ! -f ${TLS_CERT} ]; then
		$1 $2 -out ${TLS_CERT} $4
		if [ $? -ne 0 ]; then
			report_problem "Certificate for qmail TLS sessions ${TLS_CERT} was not generated. Qmail will generate it at each TLS session. It is too slow and will make high CPU load."
		fi
		echo
	fi

	if [ -f ${TLS_CERT} ]; then
		chmod 600 ${TLS_CERT} && chown qmaild:0 ${TLS_CERT}
		if [ $? -ne 0 ]; then
			report_problem "Unable to set proper permission to ${TLS_CERT} Normally, owner user and group should be root:0 and access should be 0600."
		fi
	fi
}

generate_tls_certificates()
{
	local SSL_BIN inten umask_sav

	inten="generate SSL certificates for TLS sessions"

	echo_try $inten

	SSL_BIN=${PRODUCT_ROOT_D}/admin/sbin/openssl
	if [ ! -x ${SSL_BIN} ]; then
		# try to search another appropriate OPENSSL binary
		for ssl_path in /bin /sbin /usr/bin /usr/sbin /usr/local/bin /usr/local/sbin; do
			if [ -x ${ssl_path}/openssl ]; then
				SSL_BIN=${ssl_path}/openssl
				break;
			fi
		done
	fi

	if [ ! -x ${SSL_BIN} ]; then
		report_problem "Unable to find openssl frontend binary. Certificates for qmail TLS sessions are not generated. Qmail will generate them each TLS session. It is too slow and will make high CPU load."
		return
	fi

	# store old umask and set very secure one
	umask_sav=`umask`
	umask 0077

	qmail_tls_cert ${SSL_BIN} "genrsa" rsa512.pem 512
	qmail_tls_cert ${SSL_BIN} "dhparam -2" dh512.pem 512
	qmail_tls_cert ${SSL_BIN} "dhparam -2" dh1024.pem 1024

	umask ${umask_sav}

	suc
}

remove_sendmail()
{
	sendmail_pid="`$ps | grep sendmail | grep -v grep | awk '{print $1}' -`"
	if [ "X$sendmail_pid" != "X" ]; then
		inten="shutdown running sendmail"
		echo_try $inten
		$K_TERM $sendmail_pid && suc || die $inten
	fi

	mk_backup "$sendmail" mv f

	case "$machine" in
		BSDI)
			mk_backup /etc/sendmail.cf mv f
			;;
		*)
			;;
	esac

	if [ -f "$mail_local" ]; then
		case "$machine" in
			BSD)
				chflags noschg "$mail_local" ||
				die "chflags noschg $mail_local"
				;;
			*)
				;;
		esac

		mk_backup "$mail_local" cp f

		chmod 0 "$mail_local" ||
		die "chmod 0 $mail_local"
	fi

	if [ ! -f "$sendmail" ]; then
		ln -fs "$QMAIL_ROOT_D/bin/sendmail" "$sendmail" ||
		die "ln -fs $QMAIL_ROOT_D/bin/sendmail $sendmail"
	fi
	if [ ! -f "/usr/lib/sendmail" ]; then
		ln -fs "$QMAIL_ROOT_D/bin/sendmail" "/usr/lib/sendmail" ||
		die "ln -fs $QMAIL_ROOT_D/bin/sendmail /usr/lib/sendmail"
	fi


	case "$machine" in
		linux )
		    case "$linux_distr" in
			redhat)
			    [ -f /etc/init.d/sendmail ] && chkconfig --del sendmail
#			    mk_backup "$sndml_ini" mv f
#				if [ -f "$sndml_ini" ]; then
#					inten="move $sndml_ini to $sndml_ini.${product}"
#					echo_try $inten
#					mv -f "$sndml_ini" "$sndml_ini.${product}" && suc || die $inten
#				fi
				;;
			slackware)
			    inten="remove sendmail from /etc/rc.d/rc.M"
			    echo_try $inten
			    mk_backup /etc/rc.d/rc.M cp f
			    sed -e "s/^[[:space:]]*\/usr\/sbin\/sendmail/#  \/usr\/sbin\/sendmail/g" < /etc/rc.d/rc.M > /etc/rc.d/rc.M_tmp
			    mv -f /etc/rc.d/rc.M_tmp /etc/rc.d/rc.M && suc || die $inten
			    chown 0:0 /etc/rc.d/rc.M
			    chmod 755 /etc/rc.d/rc.M
				;;
		    esac
		    ;;
		solaris)
			mk_backup "$sndml_ini" mv f
#			if [ -f "$sndml_ini" ]; then
#				inten="move $sndml_ini to $sndml_ini.${product}"
#				echo_try $inten
#				mv -f "$sndml_ini" "$sndml_ini.${product}" && suc || die $inten
#			fi
			;;
		*)
			;;
	esac

}

rsr_backup()
{
	target="$1"
	dup="$2"
	opts="$3"
	common_var=0

#	if [ -$opts "$target" ]; then
		if [ -$opts "$target.$product_suff" ]; then
			case "$dup" in
				mv)
					mv -f  $target.$product_suff $target>> $product_log 2>&1
					common_var=1
					return 1
					;;
				cp)
					cp -fp  $target.$product_suff $target>> $product_log 2>&1
					common_var=1
					return 1
					;;
				*)
					p_echo " rsr_backup: wrong option -- must be 'cp' or 'mv'"
					;;
			esac
		else
			if [ -$opts "$target.$product_suffo" ]; then
			    case "$dup" in
				mv)
					mv -f  $target.$product_suffo $target>> $product_log 2>&1
					common_var=1
					return 1
					;;
				cp)
					cp -fp  $target.$product_suffo $target>> $product_log 2>&1
					common_var=1
					return 1
					;;
				*)
					p_echo " rsr_backup: wrong option -- must be 'cp' or 'mv'"
					;;
			    esac
			fi
		fi
#	else
#		case "$opts" in
#			f|d)
#				;;
#			*)
#				p_echo " rsr_backup: wrong option -- must be 'f' or 'd'"
#				;;
#		esac
#	fi
}

restore_named()
{
	cd "$PRODUCT_ROOT_D" >> $product_log 2>&1
	[ -f /etc/sysconfig/named ] && mv -f "/etc/sysconfig/named" "/etc/sysconfig/named.bak"
	rsr_backup "/etc/sysconfig/named" mv f

	[ -L $named_conf ] && rm -f "$named_conf"
	rsr_backup "$named_conf" mv f

	[ -L $rndc_conf ] && rm -f "$rndc_conf"
	rsr_backup "$rndc_conf" mv f

	case "$machine" in
		BSD*)
			rsr_backup /etc/named.boot mv f
			;;
		linux)
		    case "$linux_distr" in
			redhat)
			    chkconfig --add named >> $product_log 2>&1
#			    std_named_sh="/etc/rc.d/init.d/named"
#			    rsr_backup $std_named_sh mv f
#				if [ -f "$std_named_sh" ]; then
#					mv -f "$std_named_sh.${product}" "$std_named_sh"
#				fi
				;;
			slackware)
			    rsr_backup /etc/rc.d/rc.inet2 cp f
				;;
		    esac
			rsr_backup /etc/named.boot mv f
			;;
		solaris)
			rsr_backup /etc/named.boot mv f
			;;
	esac
}


restore_sendmail()
{
	[ -L $sendmail ] && rm -f "$sendmail"
	[ -L /usr/lib/sendmail ] && rm -f "/usr/lib/sendmail"

	rsr_backup "$sendmail" mv f

	case "$machine" in
		BSDI)
			rsr_backup /etc/sendmail.cf mv f
			;;
		*)
			;;
	esac

	if [ -f "$mail_local" ]; then
#		case "$machine" in
#			BSD)
#				chflags schg "$mail_local" >> $product_log 2>&1
#				;;
#			*)
#				;;
#		esac

		rsr_backup "$mail_local" mv f

		chmod 4555 "$mail_local" >> $product_log 2>&1
	fi
	case "$machine" in
		linux )
		    case "$linux_distr" in
			redhat)
			    chkconfig --add sendmail >> $product_log 2>&1
#			    rsr_backup "$sndml_ini" mv f
				;;
			slackware)
			     rsr_backup /etc/rc.d/rc.M mv f
				;;
		    esac
			;;
		solaris)
			rsr_backup "$sndml_ini" mv f
			;;
		*)
			;;
	esac

}

delete_startup_scripts()
{
	cd "$PRODUCT_ROOT_D" >> $product_log 2>&1

	case "$machine" in
		BSD)
			rm -f /usr/local/etc/rc.d/${product}.sh >> $product_log 2>&1
	#		rsr_backup /etc/rc.conf cp f

#			inten="enable standard sendmail & named (within /etc/rc.conf)"
#			echo_try $inten
			sed -e 's/^sendmail_enable="NONE".*//g' \
				-e "s/^#named_enable/named_enable/g" \
				< /etc/rc.conf > /etc/rc.conf.${product}_tmp
			mv -f /etc/rc.conf.${product}_tmp /etc/rc.conf
#			suc
			;;
		BSDI)
			if [ -f /etc/rc.local.${product} ]; then
				cp -p  /etc/rc.local.${product} /etc/rc.local >> $product_log 2>&1
			fi
			;;
		linux)
		    case "$linux_distr" in
			redhat)
			    chkconfig --del ${product}
			    rm -f /etc/rc.d/init.d/${product} >> $product_log 2>&1
#			    rm -f /etc/rc.d/rc0.d/K15${product} >> $product_log 2>&1
#			    rm -f /etc/rc.d/rc1.d/K15${product} >> $product_log 2>&1
#			    rm -f /etc/rc.d/rc2.d/K15${product} >> $product_log 2>&1
#			    rm -f /etc/rc.d/rc3.d/S77${product} >> $product_log 2>&1
#			    rm -f /etc/rc.d/rc4.d/S77${product} >> $product_log 2>&1
#			    rm -f /etc/rc.d/rc5.d/S77${product} >> $product_log 2>&1
#			    rm -f /etc/rc.d/rc6.d/K15${product} >> $product_log 2>&1
				;;
			slackware)
			    if [ -f /etc/rc.d/rc.local.${product} ]; then
				cp -p  /etc/rc.d/rc.local.${product} /etc/rc.d/rc.local >> $product_log 2>&1
			    fi
				;;
		    esac
		    ;;
		solaris)
			rm -f  /etc/init.d/${product} >> $product_log 2>&1
			if [ -f /etc/rc0.d/K01${product} ]; then
				rm -f /etc/rc0.d/K01${product} >> $product_log 2>&1
			fi

			if [ -f /etc/rc1.d/K01${product} ]; then
				rm -f /etc/rc1.d/K01${product} >> $product_log 2>&1
			fi

			if [ -f /etc/rc2.d/S77${product} ]; then
				rm -f /etc/rc2.d/S77${product} >> $product_log 2>&1
			fi
			;;
	esac
}

delete_crontab()
{

	$crontab -l 2>/dev/null > /tmp/crontab.${product}

	sed -e "s/^.*\/${product}\/admin\/sbin\/statistics.*//g" \
		-e "s/^.*\/${product}\/bin\/mysqldump.*//g" \
		-e "s/^.*\/usr\/sbin\/ntpdate.*//g" \
		< /tmp/crontab.${product} > /tmp/crontab.${product}_tmp

	mv -f /tmp/crontab.${product}_tmp /tmp/crontab.${product} >> $product_log 2>&1

	$crontab /tmp/crontab.${product}  >> $product_log 2>&1

	rm -f /tmp/crontab.${product} >> $product_log 2>&1

}

remove_ftpuser()
{
	user=$1
	ftpusers_file="/etc/ftpusers"

	egrep "^$user" $ftpusers_file >> /dev/null 2>&1

	case "$?" in
			0)
				sed -e "/$user/d" < $ftpusers_file > $ftpusers_file.tmp
				mv -f $ftpusers_file.tmp $ftpusers_file
				;;
			1)
				;;
			*)
				;;
	esac

}

remove_product_users_groups()
{

#	delete users of this(unsuccessful) installation
	for i in $users_created; do
		delete_user "$i"
	done
#	delete users with group=psacln (ftpusers and webusers)
	for i in `perl -e '$gid=getgrnam("'$clients_group'"); exit if (($gid eq "") || ($gid == 0)); while(($n,$u,$g) = (getpwent)[0,2,3]) {print "$n\n" if (($gid == $g) && ($u >= 500))}'`
	do
		delete_user "$i"
		remove_ftpuser "$i"
	done
#	delete users psaadm, psaftp
#	delete_user "$admin_user"  >> "$product_log" 2>&1
#	delete_user "$anonftp_user"  >> "$product_log" 2>&1

#	delete groups of this(unsuccessful) installation
	for i in $groups_created; do
		delete_group "$i"
	done
#	delete groups psaadm, psaftp, psacln
	delete_group "$admin_group"  >> "$product_log" 2>&1
	delete_group "$anonftp_group"  >> "$product_log" 2>&1
	delete_group "$clients_group"  >> "$product_log" 2>&1

}


undo_install()
{
	p_echo
	p_echo
	p_echo "===>Removing installed ${product_bs} components ... "

	remove_modules

	get_pid $PRODUCT_ROOT_D/mysql/bin/safe_mysqld true root
	req_pid=$common_var
	if [ $req_pid -gt 1 ]; then
		$K_KILL $req_pid >> $product_log 2>&1
		sleep 2
		mysql_pid_file=$mysql_bddir/*.pid
		kill_pids $PRODUCT_ROOT_D/mysql/libexec/mysqld mysql
		sleep 2
		if [ -f "$mysql_pid_file" ]; then
			rm -f "$mysql_pid_file"
		fi
	fi

	$START_SH stop >> "$product_log" 2>&1
	sleep 2

	remove_super_server "ftp"
	remove_super_server "pop3"
	remove_super_server "pop-3"
	remove_super_server "imap4"
	remove_super_server "imap2"
	remove_super_server "imap"
	remove_super_server "smtp"
	remove_super_server "smtps"
	remove_super_server "poppassd"

	remove_product_users_groups

	if [ -f "/etc/ftpchroot" ]; then
	    sed -e "s/^@$clients_group//g" < /etc/ftpchroot > /etc/ftpchroot.tmp
	    mv -f /etc/ftpchroot.tmp /etc/ftpchroot  >> $product_log 2>&1
	fi

	if [ -f /etc/shells ]; then
	    case "$machine" in
		BSD|BSDI)
		    sed -e "s/[/]*sbin[/]*nologin//" < /etc/shells > /etc/shells.tmp
		;;
		linux|solaris)
		    sed -e "s/[/]*bin[/]*false//" < /etc/shells > /etc/shells.tmp
		;;
	    esac
	    mv -f /etc/shells.tmp /etc/shells  >> $product_log 2>&1
	fi

#	remove /tmp/.state for ASP
	remove_tmp_state

	restore_named
	restore_sendmail
#	rsr_backup $inetd_conf mv f
#	rsr_backup /etc/my.cnf mv f
#	rsr_backup /etc/named.boot mv f
	delete_startup_scripts
	delete_crontab

	rm -rf $fp_i_dir  >> $product_log 2>&1
	rsr_backup $fp_i_dir mv d
	cd /usr/local  >> $product_log 2>&1
	rm -Rf $PRODUCT_ROOT_D >> $product_log 2>&1
#	rm -Rf $product_etc
	p_echo "done"

}

smart_undo_install()
{
	[ "X$trigger_uninstall" = "X1" -o "X$do_patch" = "X1" -o "X$do_reconfigure" = "X1" ] && return
	[ "X$PLESK_INSTALLER_NOUNDO" != "X" ] && return
	# trigger_uninstall - trigger what smart_undo_install is already working now(recurrence)
	trigger_uninstall=1

	if [ "X$can_uninstall" = "X1" ]; then
		if [ "X$do_upgrade" = "X1" ]; then
			undo_upgrade
		else
			undo_install
		fi
	fi

	# put suggestions for user what to do
	call_optional_function failure_note
}
undo_upgrade()
{
	p_echo   "    The attempt to upgrade $prev_product_full"
	p_echo   "    from version $prev_version to $product_full $product_version has failed."

	if [ -f "$product_sav_tar" ]; then
	
		p_echo 

		p_echo "    Now restore contents of $product_bs from $product_sav_tar"
		p_echo 
		get_pid $PRODUCT_ROOT_D/mysql/bin/safe_mysqld true root
		req_pid=$common_var
		if [ $req_pid -gt 1 ]; then
			$K_KILL $req_pid >> $product_log 2>&1 
			sleep 2
			mysql_pid_file=$mysql_bddir/*.pid
			kill_pids $PRODUCT_ROOT_D/mysql/libexec/mysqld mysql
			sleep 2
			if [ -f "$mysql_pid_file" ]; then
				rm -f $mysql_pid_file
			fi
		fi

		$START_SH stop >> $product_log 2>&1
		sleep 2

		rm -Rf $PRODUCT_ROOT_D
		setup_super_server "smtp"
		setup_super_server "smtps"
		setup_super_server "ftp"
		setup_super_server "poppassd"
		

		cd /
        	tar xvf $product_sav_tar > /dev/null 2>&1
		case "$?" in
			0)
#				rm -f $product_sav_tar 		
			;;
			*)
				p_echo 
				p_echo 
				p_echo "ERROR: restoration of files from $product_sav_tar has failed."
				p_echo "       Please restore them manually."
				p_echo 
				p_echo "Exiting..."
				p_echo 
				exit 0
			;;
		esac
	fi
	
}
initial_conf()
{
	DEMO_VERSION="no"
	PRODNAME="psa"
	PRODUCT_NAME="Plesk"
	product_full="Plesk"
	PRODUCT_FULL_NAME="Plesk"

	product_etc="/etc/${PRODNAME}"
	prod_conf_t="/etc/psa/psa.conf"
	prodkey="$product_etc/$PRODNAME.key"

	minimal_changes="0"

	MYSQL_VERS="4.1.7"
	POSTGRESQL_VERS="7.4.6"
	EXTERNAL_PACKAGES=""
	EXTERNAL_PACKAGES_DIR=""

	BUILDER_UID="10007"

	PERL5LIB=/usr/local/psa/lib/perl5/site_perl/5.8.0
	export PERL5LIB

	product_bs=Plesk
	support_email="support@sw-soft.com"
	sales_email="sales@sw-soft.com"

	product_version="7.5.3"
	product_db_version="0753"
	product_build="75050506.13"
	conceived_os_vendor="RedHat"
	conceived_os_version="el3"
	osrels="rhel3"

	# This variable contains all versions, which is supported by
	# cumulative upgrade
	known_product_versions="20 25 50 60 70 71 75"

	prev_product="plesk"
	prev_clients_group="${prev_product}cln"

	# GID for mailman
	mm_mailgroup=12

	# plesk default password
	if [ "X$DEMO_VERSION" = "Xyes" ]; then
		PRODUCT_DEFAULT_PASSWORD="plesk"
	else
		PRODUCT_DEFAULT_PASSWORD="setup"
	fi

	my_name_0="`pwd`"
	if [ $? -ne 0 ]; then
		echo "Unable to determine current working directory."
		echo "Please fix the problem and run installation again."
		exit 1
	fi

	my_name_1="`basename ""$0""`"
	my_name_2=`echo ""$0""|sed -e "s/^\.\///"`
	echo $my_name_2 | egrep '^/' >/dev/null 2>&1
	case "$?" in
		0)
			my_name="$my_name_2"
		;;
		1)
			my_name="$my_name_0"/"$my_name_2"
		;;
	    *)
			echo "Unable to determine the name of installer."
			echo "Please fix the problem and run installation again."
			exit 1
		;;
	esac
}

read_conf()
{
	initial_conf

	if [ -s $prod_conf_t ]; then
		tmp_var=`perl -e 'undef $/; $_=<>; s/#.*$//gm;
				s/^\s*(\S+)\s*/$1=/mg;
				print' $prod_conf_t`
		eval $tmp_var
	else
		[ 0$ignore_miss_conf -ne 1 ] && p_echo "Unable to find product configuration file: $prod_conf_t"
		return 1
#		die "find product configuration file: $prod_conf_t"
	fi
	return 0

}

product_conf()
{
	initial_conf
	product_default_conf
	APACHE_ROOT="/usr/local/psa/apache"
	TRUE_BIN="$QMAIL_ROOT_D/bin/true"
	rpm_suexec_bin=/usr/sbin/psa-suexec
	MYSQL_SOCKETS="/var/lib/mysql/mysql.sock /tmp/mysql.sock /var/run/mysqld/mysqld.sock"
	parse_tomcat_conf
}

parse_tomcat_conf()
{
       T4_CF=/etc/tomcat4/tomcat4.conf
       if [ -r $T4_CF ]; then
               catalinahome=`egrep "^CATALINA_HOME=" $T4_CF 2>/dev/null | sed -e "s/CATALINA_HOME=\"\(.*\)\"/\1/"`
               if [ -n $catalinahome ]; then
                       CATALINA_HOME=$catalinahome
			if [ -f $prod_conf_t ]; then
                	       echo "===> Updating $prod_conf_t..."
        	               sed -e "s#^\\(CATALINA_HOME\\)\\([[:space:]]*\\).*#\\1\\2$CATALINA_HOME#" "$prod_conf_t" >"$prod_conf_t.tmp" || die "update $prod_conf_t"
			       mv "$prod_conf_t.tmp" "$prod_conf_t" || die "update $prod_conf_t"
	               fi
		fi
       fi
}

 #default values

product_default_conf()
{

PRODUCT_ROOT_D=/usr/local/psa
PRODUCT_RC_D=/etc/init.d
HTTPD_VHOSTS_D=/home/httpd/vhosts
HTTPD_CONF_D=/etc/httpd/conf
HTTPD_BIN_D=/usr/bin
QMAIL_ROOT_D=/var/qmail
QMAIL_MAILNAMES_D=/var/qmail/mailnames
RBLSMTPD=/usr/sbin/rblsmtpd
COURIER_IMAP_ROOT_D=/usr/lib/courier-imap
FTPD_CONF=/etc/proftpd.conf
FTPD_CONF_INC=/etc/proftpd.include
FTPD_BIN_D=/usr/bin
FTPD_VAR_D=/var/run/proftpd
OPENSSL=/usr/bin/openssl
NAMED_RUN_ROOT_D=/var/named/run-root
WEB_STAT=/usr/bin/webalizer
LOGROTATE=/usr/local/psa/logrotate/sbin/logrotate
MYSQL_VAR_D=/var/lib/mysql
MYSQL_BIN_D=/usr/bin
PGSQL_DATA_D=/var/lib/pgsql/data
PGSQL_BIN_D=/usr/bin
DUMP_D=/var/lib/psa/dumps
MAILMAN_ROOT_D=/var/mailman
MAILMAN_VAR_D=/var/mailman
PYTHON_ROOT_D=/usr
CATALINA_HOME=/var/tomcat4
DRWEB_ROOT_D=/opt/drweb
DRWEB_ETC_D=/etc/drweb

}

get_domain()
{
	host="`hostname`"
	try_dot=`echo "$host" | sed -e 's/^[^.]*\..*/\./'`
	case "$try_dot" in
		.)	domain="`echo "$host" | sed -e 's/^[^.]*\.//'`"
			host="`echo $host | sed -e 's/\..*//'`"
			;;
		*)	domain="`$get_domainname`"
			;;
	esac
	domain=`echo "$domain" | sed 's/\.$//'`
}

if [ 0$1 -gt 1 ]; then
	do_upgrade=1
else
	do_upgrade=0
fi

my_name_1="`basename ""$0""`"

if [ "X${PLESK_INSTALLER_DEBUG}" != "X" ]; then
    set -x
fi

do_rpm=1
do_qmail_rpm=1
ignore_miss_conf=1

product_log=/tmp/qmail-rom.log
#do_upgrade=0
#xinetd_conf=/etc/xinetd.conf
#inetd_conf=/etc/inetd.conf
#sendmail=/usr/sbin/sendmail
#ps="ps axww"
#K_HUP="/bin/kill -HUP"
#K_KILL="/bin/kill -KILL"
#K_TERM="/bin/kill -TERM"
#K_USR2="/bin/kill -USR2"

product_conf

read_conf;

#TRUE_BIN="/bin/true"

set_common_params
get_domain

set_qmail_perms
install_qmail
